/*
 * GasTexConverter.cpp
 *
 *  Created on: Jan 1, 2014
 *      Author: Martin Zak
 */

#include "GasTexConverter.h"

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/PDA/NPDA.h>
#include <automaton/PDA/SinglePopNPDA.h>
#include <automaton/TM/OneTapeDTM.h>

#include <set>
#include <map>
#include <typeinfo>
#include "exception/AlibException.h"

void GasTexConverter::convert(const automaton::Automaton& a, std::ostream& out) {
	a.getData().Accept((void*) &out, GasTexConverter::instance);
}

void GasTexConverter::convert(const automaton::UnknownAutomaton& a, std::ostream& out) {
	//TODO
}

void GasTexConverter::convert(const automaton::EpsilonNFA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::NFA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::DFA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::ExtendedNFA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::CompactNFA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::NPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::SinglePopNPDA& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

void GasTexConverter::convert(const automaton::OneTapeDTM& a, std::ostream& out) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state.getName();
		out << ")(,){";
		out << state.getName();
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

std::string GasTexConverter::getStackSymbols(const std::list<alphabet::Symbol>& stackSymbols) {
	if (stackSymbols.size() == 0) {
		return "$\\varepsilon$";
	}

	std::string symbols = "";
	int i=0;
	for (const auto& symbol : stackSymbols) {
		if(i++ !=0) {
			symbols +=" ";
		}
		symbols += (std::string) symbol.getData();
	}
	return symbols;
}

void GasTexConverter::printTransitionMap(const std::map<std::pair<std::string, std::string>, std::string> transitionMap, std::ostream& out) {
	for (const auto& transition : transitionMap) {
		std::string from = transition.first.first;
		std::string to = transition.first.second;
		std::string symbols = transition.second;

		if (from == to) {
			out << "\\drawloop(";
			out << from;
			out << "){";
			out << symbols;
			out << "}\n";

		} else {
			out << "\\drawedge(";
			out << from;
			out << ",";
			out << to;
			out << "){";
			out << symbols;
			out << "}\n";
		}
	}
}

void GasTexConverter::transitions(const automaton::EpsilonNFA& fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;
	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key(transition.first.first.getName(), to.getName());

			std::string symbol;
			if (transition.first.second.is<string::Epsilon>()) {
				symbol = "$\\varepsilon$";
			} else {
				symbol = (std::string) transition.first.second.get<alphabet::Symbol>();
			}

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::NFA& fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;
	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key(transition.first.first.getName(), to.getName());

			std::string symbol = (std::string) transition.first.second;

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::DFA& fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;
	for (const auto& transition : fsm.getTransitions()) {
		std::pair<std::string, std::string> key(transition.first.first.getName(), transition.second.getName());

		std::string symbol = (std::string) transition.first.second;

		auto mapIterator = transitionMap.find(key);
		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(make_pair(key, symbol));
		} else {
			mapIterator->second += ", " + symbol;
		}
	}
	printTransitionMap(transitionMap, out);
}

void GasTexConverter::transitions(const automaton::ExtendedNFA& pda, std::ostream& out) {
	//TODO
}

void GasTexConverter::transitions(const automaton::CompactNFA& pda, std::ostream& out) {
	// TODO
}

void GasTexConverter::transitions(const automaton::NPDA& pda, std::ostream& out) {
	// TODO
	/*map<pair<string, string>, string> transitionMap;

	for (auto& transition : pda.getTransitions()) {

		pair<string, string> key(transition.getFrom().getName(), transition.getTo().getName());
		auto mapIterator = transitionMap.find(key);

		string symbol = checkEpsilon(transition.getInput().getSymbol());

		symbol += ",";


		symbol += getStackSymbols(transition.getPop());
		symbol += "/";
		symbol += getStackSymbols(transition.getPush());

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(pair<pair<string, string>, string>(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);*/
}

void GasTexConverter::transitions(const automaton::SinglePopNPDA& tm, std::ostream& out) {
	// TODO
}

void GasTexConverter::transitions(const automaton::OneTapeDTM& tm, std::ostream& out) {
	// TODO
	/*map<pair<string, string>, string> transitionMap;

	for (auto& transition : tm.getTransitions()) {

		pair<string, string> key(transition.getFrom().getName(), transition.getTo().getName());
		auto mapIterator = transitionMap.find(key);

		string symbol = checkEpsilon(transition.getInput().getSymbol());
		symbol += "/";
		symbol += checkEpsilon(transition.getOutput().getSymbol());
		symbol += ",";
		symbol += (std::string[] ) { "$\\leftarrow$", "$\\rightarrow$", "$\\times$" } [transition.getShift()];

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(pair<pair<string, string>, string>(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);*/
}

const GasTexConverter GasTexConverter::instance;

void GasTexConverter::Visit(void* data, const automaton::UnknownAutomaton& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::EpsilonNFA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::NFA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::DFA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::ExtendedNFA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::CompactNFA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::NPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::SinglePopNPDA& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

void GasTexConverter::Visit(void* data, const automaton::OneTapeDTM& automaton) const {
	GasTexConverter::convert(automaton, *((std::ostream*) data));
}

