#include "FAtoLRGConverter.h"
#include <map>

#include <label/StringLabel.h>
#include <alphabet/LabeledSymbol.h>

namespace fa2rg
{

grammar::LeftRG FAtoLRGConverter::convert(const automaton::NFA& automaton)
{
    std::map<automaton::State, alphabet::Symbol> nonterminalMap;
    // step 2
    grammar::LeftRG grammar(alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("S")))));

    // step 1
    grammar.setTerminalAlphabet(automaton.getInputAlphabet());

    for(const auto& state : automaton.getStates())
    {
        alphabet::Symbol nt = alphabet::createUniqueSymbol(alphabet::Symbol(alphabet::LabeledSymbol(state.getName())), grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
        grammar.addNonterminalSymbol(nt);
        nonterminalMap.insert(std::pair<automaton::State, alphabet::Symbol>(state, nt));
    }


    // step 3 - create set of P in G
    for(const auto& transition : automaton.getTransitions())
    {
        const automaton::State& from = transition.first.first;
        const alphabet::Symbol& input = transition.first.second;

        for(const auto& to : transition.second)
        {
            // 3a
            grammar.addRule(nonterminalMap.find(to)->second, std::make_pair(nonterminalMap.find(from)->second, input));

            if(automaton.getFinalStates().count(to) > 0)
                grammar.addRule(grammar.getInitialSymbol(), std::make_pair(nonterminalMap.find(from)->second, input));


            if(automaton.getInitialStates().count(from) > 0)
            {
                grammar.addRule(nonterminalMap.find(to)->second, input);

                if(automaton.getFinalStates().count(to) > 0)
                    grammar.addRule(grammar.getInitialSymbol(), input);
            }
        }
    }

    if(automaton.getFinalStates().count(*automaton.getInitialStates().begin()) > 0)
        grammar.setGeneratesEpsilon(true);

    return grammar;
}

} /* namespace fa2rg */
