/*
 * Glushkov.cpp
 *
 *  Created on: 11. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "Glushkov.h"
#include "label/Label.h"
#include "label/StringLabel.h"
#include "label/CharacterLabel.h"
#include "label/IntegerLabel.h"
#include "label/LabelPairLabel.h"

namespace re2fa {

Glushkov::Glushkov( const regexp::UnboundedRegExp & re ) : m_re( re ) {

}

automaton::NFA Glushkov::convert( void ) {
	automaton::NFA automaton;

	// step 1
	for( auto const& symbol : m_re.getAlphabet( ) )
		automaton.addInputSymbol( symbol );

	// steps 2, 3, 4
	m_first = regexp::GlushkovTraversal::first( m_re );
	m_last = regexp::GlushkovTraversal::last( m_re );
	for( auto const& x : regexp::GlushkovTraversal::getSymbols( m_re ) )
		for( auto const& f : regexp::GlushkovTraversal::follow( m_re, x ) ) {
			m_pairs.insert( regexp::GlushkovPair( x, f ) );
			//std::cout << x.getSymbolPtr() << "|" << x.getSymbol() << "|" << x.getId() << "  +  ";
			//std::cout << f.getSymbolPtr() << "|" << f.getSymbol() << "|" << f.getId() << std::endl;
		}
	// \e in q0 check is in step 7

	// step 5
	automaton::State q0( label::Label( label::LabelPairLabel( std::make_pair( label::Label( label::CharacterLabel( 'q' ) ), label::Label( label::IntegerLabel ( 0 ) ) ) ) ) );
	automaton.addState( q0 );
	automaton.addInitialState( q0 );
	for( auto const& symbol : regexp::GlushkovTraversal::getSymbols( m_re ) ) {
		automaton::State q( label::Label( label::LabelPairLabel( std::make_pair( label::Label( label::StringLabel( symbol.getInputSymbol( ) ) ), label::Label( label::IntegerLabel ( symbol.getId( ) ) ) ) ) ) );

		m_stateMap.insert( std::make_pair( symbol, q ) );
		automaton.addState( q );
	}

	// step 6
	for( auto const& symbol : m_first ) {
		const automaton::State & q = m_stateMap.find( symbol )->second;

		automaton.addTransition( q0, symbol.getInputSymbol( ), q );
	}

	for( auto const& pair : m_pairs ) {
		const automaton::State & p = m_stateMap.find( pair.getFirst( ) )->second;
		const automaton::State & q = m_stateMap.find( pair.getSecond( ) )->second;

		automaton.addTransition( p, pair.getSecond( ).getInputSymbol( ), q );
	}

	// step 7
	for( auto const& symbol : m_last ) {
		const automaton::State & q = m_stateMap.find( symbol )->second;
		automaton.addFinalState( q );
	}

	if( m_re.containsEmptyString( ) )
		automaton.addFinalState( q0 );

	return automaton;
}

} /* namespace conversions */
