/*
 * EpsilonNFAEpsilonRemover.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "EpsilonNFAEpsilonRemover.h"

#include "../../automaton/AutomatonPropertiesFSM.h"

namespace epsilon {

automaton::NFA EpsilonNFAEpsilonRemover::remove(const automaton::NFA& origFSM)
{
    return origFSM;
}

automaton::NFA EpsilonNFAEpsilonRemover::remove( const automaton::EpsilonNFA & origFSM ) {
	automaton::NFA fsm;

	for( const auto & state : origFSM.getStates() )
		fsm.addState( state );

	for( const auto & state : origFSM.getInitialStates() )
		fsm.addInitialState( state );

	for( const auto & symbol : origFSM.getInputAlphabet() )
		fsm.addInputSymbol( symbol );

	/**
	 * Step 1 from Melichar 2.41
	 */
	for( const auto & from : origFSM.getStates( ) )
		for( const auto & fromClosure : automaton::AutomatonPropertiesFSM::epsilonClosure( origFSM, from ) )
			for( const auto & transition : origFSM.getSymbolTransitionsFromState( fromClosure ) )
				for( const auto & to : transition.second )
					fsm.addTransition( from, transition.first.second, to );

	/**
	 * Step 2 from Melichar 2.41
	 */
	std::set<automaton::State> finalStates;

	for( const auto & q : fsm.getStates( ) )
	{
		const std::set<automaton::State> & cl = automaton::AutomatonPropertiesFSM::epsilonClosure( origFSM, q );
		const std::set<automaton::State> & F = origFSM.getFinalStates( );
		std::set<automaton::State> intersect;

		set_intersection( cl.begin(), cl.end(), F.begin(), F.end(), std::inserter( intersect, intersect.begin() ) );
		if( intersect.size( ) != 0 )
			finalStates.insert( q );
	}

	for( const auto & q : finalStates )
		fsm.addFinalState( q );

	return fsm;
}

} /* namespace epsilon */
